import {
  Body,
  Controller,
  Get,
  Query,
  Put,
  Delete,
  Post,
  BadRequestException
} from '@nestjs/common';
import { ApiOperation, ApiTags } from '@nestjs/swagger';
import { Perm, definePermission } from '../auth/decorators/permission.decorator';
import { ApiSecurityAuth } from '~/common/decorators/swagger.decorator';
import { ContractService } from './contract.service';
import { ApiResult } from '~/common/decorators/api-result.decorator';
import { ContractEntity } from './contract.entity';
import { ContractDto, ContractQueryDto, ContractUpdateDto } from './contract.dto';
import { IdParam } from '~/common/decorators/id-param.decorator';
export const permissions = definePermission('app:contract', {
  LIST: 'list',
  CREATE: 'create',
  READ: 'read',
  UPDATE: 'update',
  DELETE: 'delete'
} as const);

@ApiTags('Contract - 合同')
@ApiSecurityAuth()
@Controller('contract')
export class ContractController {
  constructor(private contractService: ContractService) {}

  @Get()
  @ApiOperation({ summary: '获取合同列表' })
  @ApiResult({ type: [ContractEntity], isPage: true })
  @Perm(permissions.LIST)
  async list(@Query() dto: ContractQueryDto) {
    return this.contractService.findAll(dto);
  }

  @Get(':id')
  @ApiOperation({ summary: '获取合同信息' })
  @ApiResult({ type: ContractDto })
  @Perm(permissions.READ)
  async info(@IdParam() id: number) {
    return this.contractService.info(id);
  }

  @Post()
  @ApiOperation({ summary: '新增合同' })
  @Perm(permissions.CREATE)
  async create(@Body() dto: ContractDto): Promise<void> {
    await this.contractService.create(dto);
  }

  @Put(':id')
  @ApiOperation({ summary: '更新合同' })
  @Perm(permissions.UPDATE)
  async update(@IdParam() id: number, @Body() dto: ContractUpdateDto): Promise<void> {
    await this.contractService.update(id, dto);
  }

  @Delete(':id')
  @ApiOperation({ summary: '删除合同' })
  @Perm(permissions.DELETE)
  async delete(@IdParam() id: number): Promise<void> {
    await this.contractService.delete(id);
  }

  @Put('unlink-attachments/:id')
  @ApiOperation({ summary: '附件解除关联' })
  @Perm(permissions.UPDATE)
  async unlinkAttachments(@IdParam() id: number, @Body() {fileIds}: ContractUpdateDto): Promise<void> {
    await this.contractService.unlinkAttachments(id, fileIds);
  }

}
