import { Injectable } from '@nestjs/common';
import { InjectEntityManager, InjectRepository } from '@nestjs/typeorm';
import { MaterialsInventoryEntity } from './materials_inventory.entity';
import { EntityManager, Repository } from 'typeorm';
import {
  MaterialsInventoryDto,
  MaterialsInventoryQueryDto,
  MaterialsInventoryUpdateDto
} from './materials_inventory.dto';
import { Pagination } from '~/helper/paginate/pagination';

import { paginate } from '~/helper/paginate';

@Injectable()
export class MaterialsInventoryService {
  constructor(
    @InjectEntityManager() private entityManager: EntityManager,
    @InjectRepository(MaterialsInventoryEntity)
    private materialsInventoryRepository: Repository<MaterialsInventoryEntity>
  ) {}
  /**
   * 查询所有盘点信息
   */
  async findAll({
    page,
    pageSize
  }: MaterialsInventoryQueryDto): Promise<Pagination<MaterialsInventoryEntity>> {
    const queryBuilder = this.materialsInventoryRepository
      .createQueryBuilder('materialsInventory')
      .where('materialsInventory.isDelete = 0');

    return paginate<MaterialsInventoryEntity>(queryBuilder, {
      page,
      pageSize
    });
  }

  /**
   * 新增
   */
  async create(dto: MaterialsInventoryDto): Promise<void> {
    await this.materialsInventoryRepository.insert(dto);
  }

  /**
   * 更新
   */
  async update(id: number, data: Partial<MaterialsInventoryUpdateDto>): Promise<void> {
    await this.entityManager.transaction(async manager => {
      await manager.update(MaterialsInventoryEntity, id, {
        ...data
      });
    });
  }

  /**
   * 删除
   */
  async delete(id: number): Promise<void> {
    // 合同比较重要，做逻辑删除
    await this.materialsInventoryRepository.update(id, { isDelete: 1 });
  }

  /**
   * 获取单个合同信息
   */
  async info(id: number) {
    const info = await this.materialsInventoryRepository
      .createQueryBuilder('materialsInventory')
      .where({
        id
      })
      .andWhere('materialsInventory.isDelete = 0')
      .getOne();
    return info;
  }
}
