import { ApiProperty, PartialType } from '@nestjs/swagger';
import { IsArray, IsBoolean, IsInt, IsOptional, IsString, MinLength } from 'class-validator';

import { PagerDto } from '~/common/dto/pager.dto';

import { DictTypeEntity } from './dict-type.entity';
import { isBoolean } from 'lodash';

export class DictTypeDto extends PartialType(DictTypeEntity) {
  @ApiProperty({ description: '字典类型名称' })
  @IsString()
  @MinLength(1)
  name: string;

  @ApiProperty({ description: '字典类型code' })
  @IsString()
  @MinLength(3)
  code: string;

  @ApiProperty({ description: '状态' })
  @IsOptional()
  @IsInt()
  status?: number;

  @ApiProperty({ description: '备注' })
  @IsOptional()
  @IsString()
  remark?: string;
}

export class DictTypeQueryDto extends PagerDto {
  @ApiProperty({ description: '字典类型名称' })
  @IsString()
  @IsOptional()
  name: string;

  @ApiProperty({ description: '字典类型code' })
  @IsString()
  @IsOptional()
  code: string;

  @ApiProperty({ description: '是否用于前端store缓存' })
  @IsOptional()
  @IsBoolean()
  withItems: boolean;

  @ApiProperty({ description: '需要前端store缓存的code' })
  @IsArray()
  @IsOptional()
  storeCodes: string[];
}
